#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
    
    /*Digital inputs*/
const int selectPin1 = 2;		/*pin in GERTBOARD PD2*/
const int selectPin2 = 3;		/*pin in GERTBOARD PD3*/
    
    /*Leds to activate*/
const int LedPin1 = 8;			/*pin in GERTBOARD PB0, red*/
const int LedPin2 = 9;			/*pin in GERTBOARD PB1, red*/
const int LedPin3 = 10;			/*pin in GERTBOARD PB2, red*/
    
    /*Leds to select method*/
const int ledPin11 = 11;		/*pin in GERTBOARD PB3, blue*/
const int ledPin12 = 12;		/*pin in GERTBOARD PB4, green*/
const int ledPin13 = 13;		/*pin in GERTBOARD PB5, red*/
    
    /*Used to receive the value of selection*/
int selectMSB = 0;
int selectLSB = 0;
    
    /*Generate HIGH and LOW pins to use*/
const int pinL1 = 4;			/*pin in GERTBOARD PD4, LOW*/
const int pinL2 = 5;			/*pin in GERTBOARD PD5, LOW*/
const int pinH1 = 6;			/*pin in GERTBOARD PD6, HIGH*/
const int pinH2 = 7;			/*pin in GERTBOARD PD7, HIGH*/
    
    /*Global variables*/
unsigned char rec[16];			/*RX vector*/
unsigned char sen[16];			/*TX vector*/
unsigned char ans[2];			/*checksum auxiliar*/
unsigned char last;				/*receive the last byte*/
    
    /*Checksum union syntax*/
union checksum{
  unsigned short int soma;
  unsigned char checksum[2];
};
    
    /*Functions prototypes*/
void SerialUART();
void checaSoma();
void checaAciona();
void geraSen();
void deuErrado();
void deuCerto();

    /*Start menu*/
void setup(){
    Serial.begin(9600);			/*Starts serial communication*/
    randomSeed(analogRead(0));	/*Random seed start, based at PC0 external noise*/
    
    /*External VCC an GND*/
    pinMode(pinL1, OUTPUT);
    pinMode(pinL2, OUTPUT);
    pinMode(pinH1, OUTPUT);
    pinMode(pinH2, OUTPUT);
    digitalWrite(pinL1, LOW);
    digitalWrite(pinL2, LOW);
    digitalWrite(pinH1, HIGH);
    digitalWrite(pinH2, HIGH);
    
    /*Leds to activate*/
    pinMode(LedPin1, OUTPUT);
    pinMode(LedPin2, OUTPUT);
    pinMode(LedPin3, OUTPUT);
    digitalWrite(LedPin1, LOW);
    digitalWrite(LedPin2, LOW);
    digitalWrite(LedPin3, LOW);
    
    /*Leds to select method*/
    pinMode(ledPin11, OUTPUT);
    pinMode(ledPin12, OUTPUT);
    pinMode(ledPin13, OUTPUT);
    digitalWrite(ledPin11, LOW);
    digitalWrite(ledPin12, LOW);
    digitalWrite(ledPin13, LOW);
    
    /*Get value of communication method pins*/
    pinMode(selectPin1, INPUT);
    pinMode(selectPin2, INPUT);
}

    /*Selection menu*/
void loop(){
    /*Verification at any loop*/
    selectLSB = digitalRead(selectPin1);
    selectMSB = digitalRead(selectPin2);
    
    if(selectLSB == LOW && selectMSB == LOW){
	//UART Method
	digitalWrite(ledPin11, HIGH);
	digitalWrite(ledPin12, LOW);
	digitalWrite(ledPin13, LOW);
	SerialUART();
    }else if(selectLSB == HIGH && selectMSB == LOW){
	//ISP Method
	digitalWrite(ledPin12, HIGH);
	digitalWrite(ledPin11, LOW);
	digitalWrite(ledPin13, LOW);
    }else if(selectLSB == HIGH && selectMSB == HIGH){
	//I2C Method
	digitalWrite(ledPin13, HIGH);
	digitalWrite(ledPin11, LOW);
	digitalWrite(ledPin12, LOW);
    }
}

    /*RX Analysis*/
void SerialUART(){
    if(Serial.available()){
	last = 0;
	last = Serial.read();
    
	rec[0] = rec[1];
	rec[1] = rec[2];
	rec[2] = rec[3];
	rec[3] = rec[4];
	rec[4] = rec[5];
	rec[5] = rec[6];
	rec[6] = rec[7];
	rec[7] = rec[8];
	rec[8] = rec[9];
	rec[9] = rec[10];
	rec[10] = rec[11];
	rec[11] = rec[12];
	rec[12] = rec[13];
	rec[13] = rec[14];
	rec[14] = rec[15];
	rec[15] = last;
	
	last = 0;
    }

    if(rec[0]=='#'){
	checaAciona();
	rec[0] = 0;
    }
}

    /*Leds activation and verification*/
void checaAciona(){
    union checksum chk;
    int i;
 
    chk.soma = 0;
    for(i=0; i<=13; i++){
	chk.soma += rec[i];
    }

    ans[0] = chk.checksum[0];
    ans[1] = chk.checksum[1];
    if(ans[0] == rec[14] && ans[1] == rec[15]){
	if(rec[1] == 'L' && rec[2] == 'E' && rec[3] == 'D'){
	    int led;
	    if(rec[4] == '1'){
		led = digitalRead(LedPin1);
		if(led == HIGH){
		    digitalWrite(LedPin1, LOW);
		}else{
		    digitalWrite(LedPin1, HIGH);
		}
	    }else if(rec[4] == '2'){
		led = digitalRead(LedPin2);
		if(led == HIGH){
		    digitalWrite(LedPin2, LOW);
		}else{
		    digitalWrite(LedPin2, HIGH);
		}
	    }else if(rec[4] == '3'){
		led = digitalRead(LedPin3);
		if(led == HIGH){
		    digitalWrite(LedPin3, LOW);
		}else{
		    digitalWrite(LedPin3, HIGH);
		}
	    }
	    deuCerto();
	}else{
	    deuErrado();
	}
    }else{
	deuErrado();
    }
}

    /*Return Success*/
void deuCerto(){
    int i;
    geraSen();

    sen[1] = rec[1];
    sen[2] = rec[2];
    sen[3] = rec[3];
    sen[4] = rec[4];
    sen[5] = 'O';
    sen[6] = 'K';
    checaSoma();

    for(i=0; i<=15; i++){
	Serial.write(sen[i]);
    }
}

    /*Return Error*/
void deuErrado(){
    int i;
    geraSen();

    sen[1] = 'E';
    sen[2] = 'R';
    sen[3] = 'R';
    sen[4] = 'O';
    checaSoma();

    for(i=0; i<=15; i++){
	Serial.write(sen[i]);
    }
}

    /*Generate the answer package*/
void geraSen(){
    int i;
    sen[0] = '#';

    for(i=1; i<=15; i++){        
	sen[i] = random(65,126);
    }
}

    /*Calc the answer checksum*/
void checaSoma(){
    union checksum chk2;
    int i;

    chk2.soma = 0;
    for(i=0; i<=13; i++){
	chk2.soma += sen[i];
    }

    sen[14] = chk2.checksum[0];
    sen[15] = chk2.checksum[1];
}